#pragma once

#include "Motor.h"


/*!
 * @author Die Bitschubser
 * @date 12.06.2019
 * @file Engine.h
 * @version 2.2
 * @brief Enthaelt die Klasse Engine
 *****/

namespace STFU
{
/*!
 * Dieses Literal (User-defined literal) vereinfacht den Umgang mit der Motorgeschwindigkeit (0 - 255).
 * Die uebergebene prozentuale Geschwindigkeit (0 - 100) wird passend umgewandelt.
 * 0 steht fuer 0% und 100 fuer 100% der Motordrehgeschwindigkeit.
 *
 * Wenn ein Wert > 100 uebergeben wird, wird dieser als 100% interpretiert.
 * Außerdem wird das Ergebnis der Berechnung abgerundet, da es zu einem Integer (int) konvertiert wird.
 *
 * @exception Werte < 0 werden nicht an diese Funktion uebergeben, da der Datentyp des Parameters unsigned ist!
 *            Genauere ueberpruefungen des Wertes finden nicht statt!
 *            Das heisst, dass der negative Wert als positiv interpretiert wird und somit nicht zum erwuenschten Erfolg fuehren kann.
 */
int operator""_percent(unsigned long long int);

/*!
 * @class Engine Engine.h
 * @brief Diese Klasse kombiniert einen linken und rechten Motor zu einem Antrieb.
 *****/
class Engine
{
public:

  /*!@brief Antrieb Richtung der Motoren */
  /*!@brief Richtung: Vorwaerts */
  static const int FORWARD = 0;
  /*!@brief Richtung: Rueckwaerts */
  static const int BACKWARD = 255;

  /*!@brief Antrieb Geschwindigkeit */
  /*!@brief maximale Geschwindigkeit: 255, entspricht 100% Leistung */
  static const int MAX_SPEED = 255;
  /*!@brief halbe Geschwindigkeit: 127, entspricht 50% Leistung */
  static const int HALF_SPEED = 127;
  /*!@brief minimale Geschwindigkeit: 0, entspricht 0% Leistung */
  static const int MIN_SPEED = 0;

  /*!
   * @brief Der Defaultkonstruktor ist verboten
   */
  Engine() = delete;
  /*!
   * @brief Konstruktor mit linkem und rechtem Motor
   * @param[inout] Motor& left  Dies ist der linke Motor des Antriebs
   * @param[inout] Motor& right  Dies ist der rechte Motor des Antriebs
   */
  Engine(Motor& left, Motor& right);

  /*!
   * @brief Im Destruktor werden die verwendeten Resourcen wieder freigegeben
   */
  ~Engine();

  /*!
   * @brief Abfrage der aktuell maximalen Motorgeschwindigkeit
   * @return Die aktuelle Geschwindigkeit (0-255) der Motoren im Durchschnitt (Mittel)
   */
  int speed() const;

  /*!
   * @brief Abfrage der Motor Drehrichtung
   * @return Die aktuelle Richtung der Motoren im Durchschnitt (Mittel)<br>
   *		 ca.  0  = Fridolin faerht vorwaerts<br>
   *		 ca. 127 = Fridolin dreht sich im Stand<br>
   *         ca. 255 = Fridolin faerht rueckwaerts      
   */
  int direction() const;

  /*!
   * @brief Anhalten der Motoren
   * Die Geschwindigkeit der Motoren wird auf 0 gesetzt.
   */
  void stop();

  /*!
   * @brief Alle Motoren drehen in der uebergebenen Geschwindigkeit vorwaerts
   * @param[in] int  Geschwindigkeit die auf beide Motoren uebertragen werden soll (0 - 255)
   *
   * @see Engine::driveForward(int sL, int sR)
   */
  void driveForward(int);
  /*!
   * @brief Beide Motoren drehen vorwaerts.
   * Der linke Motor mit einer Geschwindigkeit von sL.
   * Der rechte Motor mit einer Geschwindigkeit von sR.
   *
   * @param[in] int sL  Geschwindigkeit die auf den linken Motor uebertragen werden soll (0 - 255)
   * @param[in] int sR  Geschwindigkeit die auf den rechten Motor uebertragen werden soll (0 - 255)
   *
   * @see Engine::driveForward(int)
   */
  void driveForward(int sL, int sR);

  /*!
   * @brief Alle Motoren drehen in der uebergebenen Geschwindigkeit rueckwaerts
   * @param[in] int  Geschwindigkeit die auf beide Motoren uebertragen werden soll (0 - 255)
   *
   * @see Engine::driveBackward(int sL, int sR)
   */
  void driveBackward(int);
  /*!
   * @brief Beide Motoren drehen rueckwaerts.<br>
   * Der linke Motor mit einer Geschwindigkeit von sL.<br>
   * Der rechte Motor mit einer Geschwindigkeit von sR.
   *
   * @param[in] int sL  Geschwindigkeit die auf den linken Motor uebertragen werden soll (0 - 255)
   * @param[in] int sR  Geschwindigkeit die auf den rechten Motor uebertragen werden soll (0 - 255)
   *
   * @see Engine::driveBackward(int)
   */
  void driveBackward(int sL, int sR);

  /*!
   * @brief Im Stehen nach links Drehen.<br>
   * Der rechte Motor dreht vorwaerts.<br>
   * Der linke Motor dreht rueckwaerts.
   * @param[in] int  Geschwindigkeit die auf beide Motoren uebertragen werden soll (0 - 255)
   */
  void rotateLeft(int);
  /*!
   * @brief Im Stehen nach links Drehen.<br>
   * Der rechte Motor dreht mit einer Geschwindigkeit von sR vorwaerts<br>
   * und der linke Motor dreht mit einer Geschwindigkeit von sL rueckwaerts
   * @param[in] int sL  Geschwindigkeit die auf den linken Motor uebertragen werden soll (0 - 255)
   * @param[in] int sR  Geschwindigkeit die auf den rechten Motor uebertragen werden soll (0 - 255)
   */
  void rotateLeft(int sL, int sR);

  /*!
   * @brief Beim Fahren nach links Drehen.<br>
   * Beide Motoren drehen vorwaerts.<br>
   * Der rechte Motor dreht in der uebergebenen Geschwindigkeit.<br>
   * Der linke Motor dreht in halber Geschwindigkeit des rechten Motors.
   * @param[in] int  Geschwindigkeit die auf den rechten Motor uebertragen werden soll (0 - 255)<br>
   *        Es wird empfohlen 255 zu verwenden!
   */
  void turnLeft(int);

  /*!
   * @brief Im Stehen nach rechts Drehen.<br>
   * Der linke Motor dreht mit einer Geschwindigkeit von s vorwaerts<br>
   * und der rechte Motor dreht mit der selben Geschwindigkeit rueckwaerts
   * @param[in] int  Geschwindigkeit die auf den rechten Motor uebertragen werden soll (0 - 255)
   */
  void rotateRight(int);
  /*!
   * @brief Im Stehen nach rechts Drehen.<br>
   * Der linke Motor dreht mit einer Geschwindigkeit von sL vorwaerts<br>
   * und der rechte Motor dreht mit einer Geschwindigkeit von sR rueckwaerts
   * @param[in] int sL  Geschwindigkeit die auf den linken Motor uebertragen werden soll (0 - 255)
   * @param[in] int sR  Geschwindigkeit die auf den rechten Motor uebertragen werden soll (0 - 255)
   */
  void rotateRight(int sL, int sR);

  /*!
   * @brief Beim Fahren nach rechts Drehen.<br>
   * Beide Motoren drehen vorwaerts.<br>
   * Der linke Motor dreht in der uebergebenen Geschwindigkeit.<br>
   * Der rechte Motor dreht in halber Geschwindigkeit des linken Motors.
   * @param[in] int  Geschwindigkeit die auf den linken Motor uebertragen werden soll (0 - 255)<br>
   *        Es wird empfohlen 255 zu verwenden!
   */
  void turnRight(int);

  /*!
   * @brief Abfrage des linken Motors
   * @return linker Motor
   */
  const Motor& leftMotor() const;
  /*!
   * @brief Abfrage des rechten Motors
   * @return rechter Motor
   */
  const Motor& rightMotor() const;

private:
  /*! @brief linker Motor des Antriebs */
  Motor* _leftMotor = nullptr;
  /*! @brief rechter Motor des Antriebs */
  Motor* _rightMotor = nullptr;
  /*! @brief Geschwindigkeit des linken Motor des Antriebs */
  int _leftSpeed;
  /*! @brief Geschwindigkeit des rechten Motor des Antriebs */
  int _rightSpeed;


  /*!
   * @brief Regelt die Geschwindigkeit des linken und rechten Motorobjekts
   * @param[in] int  Geschwindigkeit (0 - 255), die auf beide Motoren uebertragen werden soll
   */
  void speed(int);
  /*!
   * @brief Regelt die Geschwindigkeit des linken und rechten Motorobjekts
   * @param[in] int sL  Geschwindigkeit (0 - 255), welche auf den linken Motor uebertragen werden soll
   * @param[in] int sR  Geschwindigkeit (0 - 255), welche auf den rechten Motor uebertragen werden soll
   */
  void speed(int sL, int sR);

  /*!
   * @brief Regelt die Richtung des linken und rechten Motorobjekts
   * @param[in] int  Richtung in die beide Motoren drehen sollen
   */
  void direction(int);
  /*!
   * @brief Regelt die Richtung des linken und rechten Motorobjekts
   * @param[in] int dL  Richtung in die der linke Motor drehen soll
   * @param[in] int dR  Richtung in die der rechte Motor drehen soll
   */
  void direction(int dL, int dR);

  /*!
   * @brief Berechnung des Durchschnitts zweier Werte
   * @param[in] int  Erster Wert
   * @param[in] int  Zweiter Wert
   * @return int  Der Durchschnitt
   */
  int average(int, int) const;

  /*!
   * @brief Berechnung der Geschwindigkeit beim rueckwaerts fahren
   * @param[in] int  Geschwindigkeit
   * @return int  Die umgewandelte Geschwindigkeit
   */
  int backward(int) const;
};

}