#pragma once

#include "Sensor.h"


/*!
 * @author Die Bitschubser
 * @date 21.05.2019
 * @file GroundSensor.h
 * @version 2.0
 * @brief Enthaelt die Klasse GroundSensor
 *****/

namespace STFU
{
/*!
 * @class GroundSensor GroundSensor.h
 * @brief Diese Klasse stellt die Initialisierung und Funktionen für Bodensensoren bereit
 *****/
class GroundSensor
{
public:
	/*! @brief Maximaler Helligkeitswert */
	static const int MAX_BRIGHTNESS = 255;
	/*! @brief Minimaler Helligkeitswert */
	static const int MIN_BRIGHTNESS = 0;
	
   /*!
    * @brief Der Defaultkonstruktor ist verboten
    */
    GroundSensor() = delete;
   /*!
	* @brief GroundSensor
	* @param[in] DigitalSensor& links  Dies ist der linke Bodensensor
	* @param[in] DigitalSensor& mitte  Dies ist der rechte Bodensensor
	* @param[in] DigitalSensor& rechts  Dies ist der mittlere Bodensensor
	*/
	GroundSensor(DigitalSensor&, DigitalSensor&, AnalogSensor&);

   /*!
	* @brief Abfrage des Linken Bodensensors
	* @return Gibt den aktuellen Wert des linken Bodensensors zurück (schwarz = false, weiß = true)
	*/
	bool left() const;

   /*!
	* @brief Abfrage des Rechten Bodensensors
	* @return Gibt den aktuellen Wert des rechten Bodensensors zurück (schwarz = false, weiß = true)
	*/
	bool right() const;

   /*!
	* @brief Digitale Abfrage des Mittleren Bodensensors
	* Der Wert wird mit Hilfe eins Schwellwertes bestimmt, für eine analoge Abfrage ist middleValue() geeignet.
	* Der Schwellwert kann durch threshold(int) gesetzt werden.
	* Alle Werte, die über diesen Schwellwert liegen, werden als Weiß interpretiert, alle anderen als Schwarz.
	* @return Gibt den aktuellen Wert des mittleren Bodensensors zurück (schwarz = false, weiß = true)
	*/
	bool middle() const;
   /*!
	* @brief Analoge Abfrage des Mittleren Bodensensors
	* Die Grenzwerte der Helligkeit sind durch MIN_BRIGHTNESS und MAX_BRIGHTNESS definiert.
	* @return Gibt den aktuellen Helligkeitswert des mittleren Bodensensors zurück.
	*/
	int middleValue() const;
 
   /*!
	* @brief Setzen des Schwellwertes für bool middle()
	* @param[in] int  Wert von MIN_BRIGHTNESS (dunkel) bis MAX_BRIGHTNESS (hell)
	*/
	void threshold(int);

   /*!
	* @brief Schwellwert zur bestimmung von middle()
	* @return Wert von MIN_BRIGHTNESS (dunkel) bis MAX_BRIGHTNESS (hell)
	*/
	int threshold() const;

private:
   /*!
	* @brief Abblidung der Sensorwerte auf Helligkeitswerte
	* Die rohen Sensorwerte werden auf selbst definierte Helligkeitswerte abgebildet
	* @param[in] int  Angabe des Sensorwertes
	*/
	int convert(int) const;

	/*! @brief Schwellwert des mittleren Bodensensors */
	int m_threshold = 0.5 * MAX_BRIGHTNESS;

	/*! @brief linker digitaler Bodensensor */
	DigitalSensor* _left = nullptr;
	/*! @brief rechter digitaler Bodensensor */
	DigitalSensor* _right = nullptr;
	/*! @brief mittlerer analoger Bodensensor */
	AnalogSensor* _middle = nullptr;
};

}